﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StoragesContracts;
using LawFirmContracts.ViewModels;
using LawFirmDatabaseImplement.Models;

namespace LawFirmDatabaseImplement.Implements
{
    public class BlankStorage : IBlankStorage
    {
        public List<BlankViewModel> GetFullList()
        {
            using var context = new LawFirmDatabase();
            return context.Blanks
            .Select(x => x.GetViewModel)
           .ToList();
        }
        public List<BlankViewModel> GetFilteredList(BlankSearchModel
       model)
        {
            if (string.IsNullOrEmpty(model.BlankName))
            {
                return new();
            }
            using var context = new LawFirmDatabase();
            return context.Blanks
            .Where(x => x.BlankName.Contains(model.BlankName))
           .Select(x => x.GetViewModel)
           .ToList();
        }
        public BlankViewModel? GetElement(BlankSearchModel model)
        {
            if (string.IsNullOrEmpty(model.BlankName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new LawFirmDatabase();
            return context.Blanks
            .FirstOrDefault(x =>
           (!string.IsNullOrEmpty(model.BlankName) && x.BlankName ==
           model.BlankName) ||
            (model.Id.HasValue && x.Id == model.Id))
            ?.GetViewModel;
        }
        public BlankViewModel? Insert(BlankBindingModel model)
        {
            var newBlank = Blank.Create(model);
            if (newBlank == null)
            {
                return null;
            }
            using var context = new LawFirmDatabase();
            context.Blanks.Add(newBlank);
            context.SaveChanges();
            return newBlank.GetViewModel;
        }
        public BlankViewModel? Update(BlankBindingModel model)
        {
            using var context = new LawFirmDatabase();
            var component = context.Blanks.FirstOrDefault(x => x.Id ==
           model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            context.SaveChanges();
            return component.GetViewModel;
        }
        public BlankViewModel? Delete(BlankBindingModel model)
        {
            using var context = new LawFirmDatabase();
            var element = context.Blanks.FirstOrDefault(rec => rec.Id ==
           model.Id);
            if (element != null)
            {
                context.Blanks.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
