﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StoragesContracts;
using LawFirmContracts.ViewModels;
using LawFirmListImplement.Models;

namespace LawFirmListImplement.Implements
{
    public class DocumentStorage : IDocumentStorage
    {
        private readonly DataListSingleton _source;
        public DocumentStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public List<DocumentViewModel> GetFullList()
        {
            var result = new List<DocumentViewModel>();
            foreach (var blank in _source.Documents)
            {
                result.Add(blank.GetViewModel);
            }
            return result;
        }
        public List<DocumentViewModel> GetFilteredList(DocumentSearchModel model)
        {
            var result = new List<DocumentViewModel>();
            if (string.IsNullOrEmpty(model.DocumentName))
            {
                return result;
            }
            foreach (var component in _source.Documents)
            {
                if (component.DocumentName.Contains(model.DocumentName))
                {
                    result.Add(component.GetViewModel);
                }
            }
            return result;
        }
        public DocumentViewModel? GetElement(DocumentSearchModel model)
        {
            if (string.IsNullOrEmpty(model.DocumentName) && !model.Id.HasValue)
            {
                return null;
            }
            foreach (var component in _source.Documents)
            {
                if ((!string.IsNullOrEmpty(model.DocumentName) &&
                component.DocumentName == model.DocumentName) ||
                (model.Id.HasValue && component.Id == model.Id))
                {
                    return component.GetViewModel;
                }
            }
            return null;
        }
        public DocumentViewModel? Insert(DocumentBindingModel model)
        {
            model.Id = 1;
            foreach (var blank in _source.Documents)
            {
                if (model.Id <= blank.Id)
                {
                    model.Id = blank.Id + 1;
                }
            }
            var newDocument = Document.Create(model);
            if (newDocument == null)
            {
                return null;
            }
            _source.Documents.Add(newDocument);
            return newDocument.GetViewModel;
        }
        public DocumentViewModel? Update(DocumentBindingModel model)
        {
            foreach (var blank in _source.Documents)
            {
                if (blank.Id == model.Id)
                {
                    blank.Update(model);
                    return blank.GetViewModel;
                }
            }
            return null;
        }
        public DocumentViewModel? Delete(DocumentBindingModel model)
        {
            for (int i = 0; i < _source.Documents.Count; ++i)
            {
                if (_source.Documents[i].Id == model.Id)
                {
                    var element = _source.Documents[i];
                    _source.Documents.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}
